/*       
*Code to interface PoKeysOC16 with Arduino via serial monitor.
*Type help: to show accepted commands.
*It uses ShiftOut function. Latch, data and clock pins can be any free Arduino pin.
*OC16 uses only 3 pins and gives you 16 outputs.
*/

int latchPin = 9;
int dataPin = 8;
int clockPin = 10;
int select = 0;
char buff[8];
long data;
int left, right, value;
String input;

void setup() {
  // put your setup code here, to run once:
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  Serial.begin(9600);
  Serial.println("Write 'help:' to show accepted commands");
}

void loop() {
  // put your main code here, to run repeatedly:
  //Show command list
  if(input == "help"){
    help();
  }
  //Select module
  else if(input == "select"){
    select = Serial.parseInt();
    if(select > 2 | select < 1){
      Serial.println("Error, select module again");
    }
    else{
      Serial.print("You have selected module number: ");
      Serial.println(select);
    }
    input = "";
  }
  //Set data
  else if(input == "setData"){
    for(int i = 0; i < 8; i++){
      buff[i] = Serial.read();
      delay(30);
    }
    data = strtol(buff, NULL, 2);
    setData(select, data);
  }
  //Test function 1
  else if(input == "test1"){
    test1();
  }
  //Test function 2
  else if(input == "test2"){
    test2();
  }
  //Reset outputs
  else if(input == "reset"){
    reset();
  } 
}
//Functions:
//Help function for printing commands
void help(void){
  Serial.println("Following commands are accepted:");
  Serial.println("-reset: - resets the number of modules and writes 0 to outputs, should be set at the beginnging");
  Serial.println("-select:num - selects the module, for example:");
  Serial.println("              select:1 selects the first module");
  Serial.println("Please note that the device consists of two 8 bit modules,");
  Serial.println("Module 1 and 2 are on the same device, 1 is from ocA to ocH and 2 is from ocI to ocP \n");
  Serial.println("-setData:bin - write 1 or 0 to outputs, first the module must be selected with 'select:num'");
  Serial.println("               command, input is in binary form, for example: setData:11110000 will set");
  Serial.println("               the first 4 outputs HIGH and the second 4 outputs LOW");
  Serial.println("-test1: - test function 1");
  Serial.println("-test2: - test function 2");
  Serial.println("-stop: - stops the test");
  input = "";
}
//Test function 1
void test1(){
  Serial.println("Test1 is running, write 'stop:' to stop it");
  while(input != "stop"){
    for(int i = 0; i < 16; i++){
      int x = 1 << i;
      digitalWrite(latchPin, LOW);
      shiftOut(dataPin, clockPin, MSBFIRST, x >> 8);
      shiftOut(dataPin, clockPin, MSBFIRST, x);
      digitalWrite(latchPin, HIGH);
      delay(200);
      //check stop condition
      serialEvent();
      if (input == "stop"){
        break;
      }
    }
  }
  Serial.println("Test stopped");
  input = "";
}
//Test function 2
void test2(){
  Serial.println("Test2 is running, write 'stop:' to stop it");
  while(input != "stop"){
    for(int i = 1; i > 0 ;i++){
      digitalWrite(latchPin, LOW);
      shiftOut(dataPin, clockPin, MSBFIRST, i >> 8);
      shiftOut(dataPin, clockPin, MSBFIRST, i);
      digitalWrite(latchPin, HIGH);
      delay(50);
      //check stop condition
      serialEvent();
      if (input == "stop"){
        break;
      }
    }
  }
  Serial.println("Test stopped");
  input = "";
}
//Reset function, writes all outputs LOW, should be called at the begining
void reset(){
  Serial.println("All outputs have been reset to LOW");
  int x = 0x0000;
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, x >> 8);
  shiftOut(dataPin, clockPin, MSBFIRST, x);
  digitalWrite(latchPin, HIGH);
  input = "";
}
//Set data
void setData(int x, int y){
  int mask = 0xFFFF;
  int store;
  int a;
  int b;
  Serial.print("Pin states on module ");
  Serial.print(x);
  Serial.println(" are");
  Serial.println(y, BIN);
  switch(x){
    //First module
    case 1:  
      value = mask & ((right << 8) | y);
      digitalWrite(latchPin, LOW);
      shiftOut(dataPin, clockPin, LSBFIRST, value >> 8);
      shiftOut(dataPin, clockPin, LSBFIRST, value);
      digitalWrite(latchPin, HIGH);
      left = y;
      break;
    //Second module
    case 2:
      value = mask & ((y << 8) | left);
      digitalWrite(latchPin, LOW);
      shiftOut(dataPin, clockPin, LSBFIRST, value >> 8);
      shiftOut(dataPin, clockPin, LSBFIRST, value);
      digitalWrite(latchPin, HIGH);
      right = y;
      break;
  }
  input = "";
}
//Read input from serial monitor  
void serialEvent(){
  if(Serial.available() > 0){
    input = (String)Serial.readStringUntil(':');
    delay(100);
  }
}
