/*   
*Code to interface PoSensors with Arduino via serial monitor.
*Type read to display sensor data.
*All sensors use I2C interface.
*If there are errors check if there are some numbers written at the bottom (line 183) and remove them.
*/

#include "Wire.h";
//device address = {0x23, 0x4C, 0x48, 0x40, 0x68};
//light, accelometer, temperature, humidity and temperature, voltage
String command;  
void setup() {
  // put your setup code here, to run once:
Wire.begin();
Serial.begin(9600);
Serial.println("Type 'read' to display sensor values");
}

void loop() {
  // put your main code here, to run repeatedly:
//Read command
if(Serial.available() > 0){
  command = (String)Serial.readStringUntil('d');
  delay(100);
}
//Print values
if (command == "rea"){
  lm75bRead();
  bh1750Read();
  sht21Read();
  //mcp3425Read();
  mma7660Read();
  delay(100);
  command = "";
  }
}

//Functions:
//LM75B
int lm75bRead(){
  int a;
  double temp;
  Wire.requestFrom(0x48, 2);
  if (Wire.available() == 2){
    a = (Wire.read() << 8); //Read MSB
    a += Wire.read(); //Read LSB
    a = (a >> 5);//5 LSB are not used so shift 5 to right
    temp = a * 0.125;//Convert to Celsius
  }
  delay(100);
  //Print temperature
  Serial.print("Temperature: ");
  Serial.print(temp);
  Serial.println(" Celsius");
}

//BH1750
int bh1750Read(){
  unsigned int light;
  Wire.beginTransmission(0x23);
  Wire.write(0x11);
  Wire.endTransmission();
  delay(200);
  Wire.requestFrom(0x23, 2);
  if (Wire.available() == 2){
    light = (Wire.read() << 8); //Read MSB
    light += Wire.read(); //Read LSB
  }
  delay(100);
  Serial.print("Light: ");
  Serial.print(light);
  Serial.println(" lx");
}

//SHT21
int sht21Read(){
  int tempM, tempL;
  int humM, humL;
  double temp, hum;
  //Measure temperature
  Wire.beginTransmission(0x40);
  Wire.write(0xF3);
  Wire.endTransmission();
  delay(100);
  Wire.requestFrom(0x40, 2);
  delay(100);
  //Store data
  tempM = (Wire.read() << 8);//MSB
  tempL = Wire.read();//LSB
  temp = tempM + (tempL >> 2);
  temp = -46.85 + (175.72 * temp / 65536); //Convert result
  //Print temperature
  Serial.print("Temperature: ");
  Serial.print(temp);
  Serial.println("Celsius");
  //Measure humidity
  Wire.beginTransmission(0x40);
  Wire.write(0xF5);
  Wire.endTransmission();
  delay(100);
  Wire.requestFrom(0x40, 2);
  delay(100);
  //Store data)
  humM = (Wire.read() << 8);//MSB
  humL = Wire.read();//LSB
  hum = humM + (humL >> 2);
  hum = -6 + (125 * hum / 65536);
  //Print humidity
  Serial.print("Relative humidity: ");
  Serial.print(hum);
  Serial.println("%");
}

//MCP3425 NOT WORKING!!
int mcp3425Read(){
  double voltage, LSB, code;
  int sign, store;
  //Set 16 bit sample rate, continuous conversion mode and 1x PGA gain
  Wire.beginTransmission(0x68);
  Wire.write(0x18);
  Wire.endTransmission();
  delay(100);
  Wire.requestFrom(0x68, 2);
  delay(100);
  //Store data
  store = (Wire.read() << 8);
  Serial.println(store, BIN);
  sign = store >> 15; //Store sign bit, 0 positive, 1 negative;
  //sign = sign & 0x1; -//Keep only LSB;
  store += Wire.read();
  Serial.println(store, BIN);
  Serial.print("Store:");
  Serial.println(sign, BIN);
  LSB = (2 * 2.048) / 65536;
  //Print data
  if (sign == 0){
    code = 32768 * 1 * store / 2.048;
    voltage = code * (LSB / 1); //PGA = 1;
    Serial.print("Input voltage is: ");
    Serial.print(voltage);
    Serial.println(" V");
    Serial.println(store, BIN);
    }
  else if (sign == 1){
    store = ~store;
    code = 32768 * 1 * store / 2.048;
    voltage = store * (LSB / 1); //PGA = 1;
    Serial.print("Input voltage is: -");
    Serial.print(voltage);
    Serial.println(" V");
    }
}
  
//MMA7660
int mma7660Read(){
  double x, y, z;
  //Set active mode
  Wire.beginTransmission(0x4C);
  Wire.write(0x07);
  Wire.write(0x01);
  Wire.endTransmission();
  delay(100);
  //Read data
  Wire.beginTransmission(0x4C);
  Wire.write(0x00);
  Wire.endTransmission();
  delay(100);
  Wire.requestFrom(0x4C, 3);
  x = Wire.read();
  y = Wire.read();
  z = Wire.read();
  //Calculate and print data:
  //Calculate X
  if (x > 31){
    x = (x - 63) * 0.047;
    }
  else {
    x = x * 0.047;
  }
  //Calculate Y
  if (y > 31){
    y = (y - 63) * 0.047;
    }
  else {  
    y = y * 0.047;
  }
  //Calculate Z
  if (z > 31){
    z = (z - 63) * 0.047;
    }
  else {
    z = z * 0.047;
  }
  //Print data
  Serial.print("X axis: ");
  Serial.print(x);
  Serial.println(" G");
  Serial.print("Y axis: ");
  Serial.print(y);
  Serial.println(" G");
  Serial.print("Z axis: ");
  Serial.print(z);
  Serial.println(" G");
}
